<?php

/**
 * @file
 * This file contains the callbacks for the default order panes supplied with
 * Ubercart and their corresponding helper functions.
 *
 * Order panes are defined using hook_uc_order_pane() and use a callback to
 * handle the different processes involved in order viewing/editing. The
 * default order panes are defined in uc_order_order_pane() in uc_order.module.
 */

use Drupal\Component\Utility\SafeMarkup;
use Drupal\Component\Utility\Xss;
use Drupal\Core\Ajax\AjaxResponse;
use Drupal\Core\Ajax\PrependCommand;
use Drupal\Core\Ajax\ReplaceCommand;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Render\Element;
use Drupal\Core\Link;
use Drupal\Core\Url;
use Drupal\uc_order\Entity\OrderProduct;
use Drupal\uc_order\Entity\OrderStatus;

/**
 * Handles the "Print button" order pane.
 */
function uc_order_pane_print_button($op, $order, &$form = NULL, FormStateInterface $form_state = NULL) {
  switch ($op) {
    case 'customer':
      if ($order->access('invoice')) {
        $build = array(
          '#type' => 'link',
          '#title' => t('Click to open a window with a printable invoice.'),
          '#url' => Url::fromRoute('uc_order.user_invoice_print', ['user' => $order->getOwnerId(), 'uc_order' => $order->id()], array(
            'attributes' => array(
              'onclick' => "window.open(this.href, '" . t('Invoice') . "', 'toolbar=0,scrollbars=1,location=0,statusbar=0,menubar=0,resizable=1,width=600,height=480,left=50,top=50'); return false;",
            ),
          )),
        );
        return $build;
      }
  }
}

/**
 * Handles the "Ship to" order pane.
 */
function uc_order_pane_ship_to($op, $order, &$form = NULL, FormStateInterface $form_state = NULL) {
  switch ($op) {
    case 'customer':
      if (!$order->isShippable()) {
        return;
      }
    case 'view':
      $address = $order->getAddress('delivery');
      $build = array('#markup' => $address . '<br />' . $address->phone);
      return $build;

    case 'edit-form':
      $form['ship_to']['address'] = array(
        '#type' => 'uc_address',
        '#parents' => ['ship_to'],
        '#default_value' => $order->getAddress('delivery'),
        '#required' => FALSE,
      );
      return $form;

    case 'edit-theme':
      $output = '<div class="order-pane-icons">';
      $output .= ' <img src="' . base_path() . drupal_get_path('module', 'uc_store')
                . '/images/address_book.gif" alt="' . t('Select from address book.') . '" '
                . 'title="' . t('Select from address book.') . '" onclick="load_address_select(' . $form['order_uid']['#value'] . ', \'#delivery_address_select\', \'delivery\');" />';
      $output .= ' <img src="' . base_path() . drupal_get_path('module', 'uc_store')
                 . '/images/copy.gif" alt="' . t('Copy billing information.') . '" title="'
                 . t('Copy billing information.') . '" onclick="uc_order_copy_billing_to_shipping();" />';
      $output .= '</div>';
      $output .= '<div id="delivery_address_select"></div>';
      return $output . drupal_render($form['ship_to']);

    case 'edit-process':
      $address = $order->getAddress('delivery');
      foreach ($form_state->getValue('ship_to') as $key => $value) {
        if (uc_address_field_enabled($key)) {
          $address->$key = $value;
        }
      }
      $order->setAddress('delivery', $address);
  }
}

/**
 * Handles the "Bill to" order pane.
 */
function uc_order_pane_bill_to($op, $order, &$form = NULL, FormStateInterface $form_state = NULL) {
  switch ($op) {
    case 'view':
    case 'customer':
      $address = $order->getAddress('billing');
      $build = array('#markup' => $address . '<br />' . $address->phone);
      return $build;

    case 'edit-form':
      $form['bill_to']['address'] = array(
        '#type' => 'uc_address',
        '#parents' => ['bill_to'],
        '#default_value' => $order->getAddress('billing'),
        '#required' => FALSE,
      );
      return $form;

    case 'edit-theme':
      $output = '<div class="order-pane-icons">';
      $output .= ' <img src="' . base_path() . drupal_get_path('module', 'uc_store')
                . '/images/address_book.gif" alt="' . t('Select from address book.') . '" '
                . 'title="' . t('Select from address book.') . '" onclick="load_address_select(' . $form['order_uid']['#value'] . ', \'#billing_address_select\', \'billing\');" />';
      $output .= ' <img src="' . base_path() . drupal_get_path('module', 'uc_store')
               . '/images/copy.gif" alt="' . t('Copy shipping information.') . '" title="'
               . t('Copy shipping information.') . '" onclick="uc_order_copy_shipping_to_billing();" />';
      $output .= '</div>';
      $output .= '<div id="billing_address_select"></div>';
      return $output . drupal_render($form['bill_to']);

    case 'edit-process':
      $address = $order->getAddress('billing');
      foreach ($form_state->getValue('bill_to') as $key => $value) {
        if (uc_address_field_enabled($key)) {
          $address->$key = $value;
        }
      }
      $order->setAddress('billing', $address);
  }
}

/**
 * Handles the "Customer Info" order pane.
 */
function uc_order_pane_customer($op, $order, &$form = NULL, FormStateInterface $form_state = NULL) {
  switch ($op) {
    case 'view':
      if ($order->getOwnerId()) {
        $build['uid'] = array(
          '#type' => 'link',
          '#prefix' => t('Customer number') . ': ',
          '#suffix' => '<br />',
          '#title' => $order->getOwnerId(),
          '#url' => Url::fromRoute('entity.user.canonical', ['user' => $order->getOwnerId()]),
        );
      }
      $build['primary_email'] = array(
        '#type' => 'link',
        '#prefix' => t('E-mail address') . ': ',
        '#title' => $order->getEmail(),
        '#url' => Url::fromUri('mailto:' . $order->getEmail()),
      );
      return $build;

    case 'edit-form':
      $form['customer']['uid'] = array(
        '#type' => 'hidden',
        '#default_value' => $order->getOwnerId(),
      );
      $form['customer']['uid_text'] = array(
        '#type' => 'textfield',
        '#title' => t('Customer number'),
        '#default_value' => $order->getOwnerId(),
        '#maxlength' => 10,
        '#size' => 10,
        '#disabled' => TRUE,
      );
      $form['customer']['primary_email'] = array(
        '#type' => 'email',
        '#title' => t('E-mail address'),
        '#default_value' => $order->getEmail(),
        '#maxlength' => 64,
        '#size' => 32,
      );
      return $form;

    case 'edit-theme':
      $output = '<div class="order-pane-icons">';
      $output .= ' <img src="' . base_path() . drupal_get_path('module', 'uc_store')
                . '/images/order_view.gif" alt="' . t('Search for an existing customer.') . '" '
                . 'title="' . t('Search for an existing customer.') . '" onclick="load_customer_search();" />';
      $output .= ' <img src="' . base_path() . drupal_get_path('module', 'uc_store')
                . '/images/menu_customers_small.gif" alt="' . t('Create a new customer.') . '" '
                . 'title="' . t('Create a new customer.') . '" onclick="load_new_customer_form();" />';
      $output .= '</div>';
      $output .= '<div id="customer-select"></div>';
      $output .= drupal_render($form['customer']);
      return $output;

    case 'edit-process':
      $order->setOwnerId($form_state->getValue('uid'));
      $order->setEmail($form_state->getValue('primary_email'));
  }
}

/**
 * Handles the "Products" order pane.
 */
function uc_order_pane_products($op, $order, &$form = NULL, FormStateInterface $form_state = NULL) {
  switch ($op) {
    case 'view':
    case 'customer':
      $build = array(
        '#type' => 'table',
        '#attributes' => array('class' => array('order-pane-table')),
        '#header' => array(
          'qty' => array(
            'data' => t('Quantity'),
            'class' => array('qty'),
          ),
          'product' => array(
            'data' => t('Product'),
            'class' => array('product'),
          ),
          'model' => array(
            'data' => t('SKU'),
            'class' => array('sku'),
          ),
          'cost' => array(
            'data' => t('Cost'),
            'class' => array('cost'),
          ),
          'price' => array(
            'data' => t('Price'),
            'class' => array('price'),
          ),
          'total' => array(
            'data' => t('Total'),
            'class' => array('price'),
          ),
        ),
        '#empty' => t('This order contains no products.'),
      );

      $account = \Drupal::currentUser();
      if (!$account->hasPermission('administer products')) {
        unset($build['#header']['cost']);
      }

      // @todo Replace with Views.
      foreach ($order->products as $id => $product) {
        $build[$id]['qty'] = array(
          '#theme' => 'uc_qty',
          '#qty' => $product->qty->value,
          '#cell_attributes' => array('class' => array('qty')),
        );

        if ($product->nid->entity && $product->nid->entity->access('view')) {
          $title = Link::createFromRoute($product->title->value, 'entity.node.canonical', ['node' => $product->nid->target_id])->toString();
        }
        else {
          $title = $product->title->value;
        }
        $build[$id]['product'] = array(
          '#markup' => $title . uc_product_get_description($product),
          '#cell_attributes' => array('class' => array('product')),
        );
        $build[$id]['model'] = array(
          '#markup' => $product->model->value,
          '#cell_attributes' => array('class' => array('sku')),
        );
        if ($account->hasPermission('administer products')) {
          $build[$id]['cost'] = array(
            '#theme' => 'uc_price',
            '#price' => $product->cost->value,
            '#cell_attributes' => array('class' => array('cost')),
          );
        }
        $build[$id]['price'] = array(
          '#theme' => 'uc_price',
          '#price' => $product->price->value,
          '#suffixes' => array(),
          '#cell_attributes' => array('class' => array('price')),
        );
        $build[$id]['total'] = array(
          '#theme' => 'uc_price',
          '#price' => $product->price->value * $product->qty->value,
          '#suffixes' => array(),
          '#cell_attributes' => array('class' => array('total')),
        );
//        $build[$id][$field]['#wrapper_attributes']['class'] = $build['#header'][$field]['class'];
      }

      return $build;

    case 'edit-form':
      $form['add_product_button'] = array(
        '#type' => 'submit',
        '#value' => t('Add product'),
        '#submit' => array('uc_order_pane_products_select'),
        '#ajax' => array(
          'callback' => 'uc_order_pane_products_ajax_callback',
          'wrapper' => 'product-controls',
        ),
      );
      $form['add_blank_line_button'] = array(
        '#type' => 'submit',
        '#value' => t('Add blank line'),
        '#submit' => array('uc_order_edit_products_add_blank'),
        '#ajax' => array(
          'callback' => 'uc_order_pane_products_ajax_callback',
          'wrapper' => 'product-controls',
        ),
      );

      $form['product_controls'] = array(
        '#tree' => TRUE,
        '#prefix' => '<div id="product-controls">',
        '#suffix' => '</div>',
      );

      $controls = array();

      if ($form_state->has('products_action')) {
        switch ($form_state->get('products_action')) {
          case 'select':
            $controls = uc_order_product_select_form($form['product_controls'], $form_state, $order);
            break;
          case 'add_product':
            $controls = uc_order_add_product_form($form['product_controls'], $form_state, $order, $form_state->get('node'));
            break;
        }
      }

      $form['product_controls'] += $controls;

      $form += uc_order_edit_products_form($form, $form_state, $order->products);

      return $form;

    case 'edit-theme':
      $output = drupal_render($form['add_product_button']);
      $output .= drupal_render($form['add_blank_line_button']);
      $output .= drupal_render($form['product_controls']);
      $output .= drupal_render($form['products']);

      return $output;
  }
}

/**
 * Form to choose a product to add to the order.
 *
 * @ingroup forms
 */
function uc_order_product_select_form($form, FormStateInterface $form_state, $order) {
  $options = $form_state->get('product_select_options');
  $ajax = array(
    'callback' => 'uc_order_pane_products_ajax_callback',
    'wrapper' => 'product-controls',
  );

  $form['nid'] = array(
    '#type' => 'select',
    '#options' => $options,
    '#size' => 7,
    '#ajax' => $ajax + array(
      'event' => 'dblclick',
      'trigger_as' => array(
        'name' => 'op',
        'value' => t('Select'),
      ),
    ),
  );
  $form['product_search'] = array(
    '#type' => 'textfield',
    '#title' => t('Search by name or model/SKU (* is the wildcard)'),
  );

  $form['actions'] = array('#type' => 'actions');
  $form['actions']['select'] = array(
    '#type' => 'submit',
    '#value' => t('Select'),
    '#validate' => array('uc_order_product_select_form_validate'),
    '#submit' => array('uc_order_pane_products_add'),
    '#ajax' => $ajax,
    '#weight' => 0,
  );
  $form['actions']['search'] = array(
    '#type' => 'submit',
    '#value' => t('Search'),
    '#submit' => array('uc_order_pane_products_select'),
    '#ajax' => $ajax,
    '#weight' => 1,
  );
  $form['actions']['close'] = array(
    '#type' => 'submit',
    '#value' => t('Close'),
    '#submit' => array('uc_order_pane_products_close'),
    '#ajax' => $ajax,
    '#weight' => 2,
  );

  return $form;
}

/**
 * Validation handler for uc_order_product_select_form().
 */
function uc_order_product_select_form_validate($form, FormStateInterface $form_state) {
  if ($form_state->isValueEmpty(['product_controls', 'nid'])) {
    $form_state->setErrorByName('product_controls][nid', t('Please select a product.'));
  }
}

/**
 * Sets the quantity and attributes of a product added to the order.
 *
 * @see uc_order_add_product_form()
 * @ingroup forms
 */
function uc_order_add_product_form($form, FormStateInterface $form_state, $order, $node) {
  $data = array();
  if ($form_state->hasValue(['product_controls', 'qty'])) {
    $data += \Drupal::moduleHandler()->invokeAll('uc_add_to_cart_data', array($form_state->getValue('product_controls')));
  }
  if (!empty($node->data) && is_array($node->data)) {
    $data += $node->data;
  }
  $node = uc_product_load_variant(intval($form_state->getValue(['product_controls', 'nid'])), $data);
  $form['title'] = array(
    '#markup' => '<h3>' . $node->label() . '</h3>',
  );
  $form['nid'] = array(
    '#type' => 'hidden',
    '#value' => $node->id(),
  );
  $form['qty'] = array(
    '#type' => 'uc_quantity',
    '#title' => t('Quantity'),
    '#default_value' => 1,
  );
  $form['actions'] = array('#type' => 'actions');
  $form['actions']['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Add to order'),
    '#submit' => array('uc_order_edit_products_add'),
    '#ajax' =>  array(
      'callback' => 'uc_order_pane_products_ajax_callback',
      'wrapper' => 'product-controls',
    ),
  );
  $form['actions']['cancel'] = array(
    '#type' => 'submit',
    '#value' => t('Cancel'),
    '#submit' => array('uc_order_pane_products_select'),
    '#ajax' =>  array(
      'callback' => 'uc_order_pane_products_ajax_callback',
      'wrapper' => 'product-controls',
    ),
    '#limit_validation_errors' => array(),
  );
  $form['node'] = array(
    '#type' => 'value',
    '#value' => $node,
  );

  uc_form_alter($form, $form_state, __FUNCTION__);

  return $form;
}

/**
 * Form to allow ordered products' data to be changed.
 */
function uc_order_edit_products_form($form, FormStateInterface $form_state, $products) {
  $form['products'] = array(
    '#type' => 'table',
    '#tree' => TRUE,
    '#header' => array(t('Remove'), t('Quantity'), t('Name'), t('SKU'), t('Weight'), t('Units'), t('Cost'), t('Price')),
    '#attributes' => array('id' => 'order-edit-products', 'class' => array('order-pane-table')),
    '#empty' => t('This order contains no products.'),
  );
  $form['data'] = array(
    '#tree' => TRUE,
    '#parents' => array('products'),
  );
  foreach ($products as $i => $product) {
    $form['products'][$i]['remove'] = array(
      '#type' => 'image_button',
      '#title' => t('Remove this product.'),
      '#name' => "products[$i][remove]",
      '#src' => drupal_get_path('module', 'uc_store') . '/images/error.gif',
      '#button_type' => 'remove',
      '#submit' => array('uc_order_edit_products_remove', '::submitForm'),
      '#return_value' => $product->order_product_id->value,
    );
    $form['data'][$i]['order_product_id'] = array(
      '#type' => 'hidden',
      '#value' => $product->order_product_id->value,
    );
    $form['data'][$i]['nid'] = array(
      '#type' => 'hidden',
      '#value' => $product->nid->target_id,
    );
    $form['products'][$i]['qty'] = array(
      '#type' => 'uc_quantity',
      '#title' => t('Quantity'),
      '#title_display' => 'invisible',
      '#default_value' => $product->qty->value,
    );
    $form['products'][$i]['title'] = array(
      '#type' => 'textfield',
      '#title' => t('Title'),
      '#title_display' => 'invisible',
      '#default_value' => $product->title->value,
      '#size' => 30,
      '#maxlength' => 255,
    );
    $form['products'][$i]['model'] = array(
      '#type' => 'textfield',
      '#title' => t('SKU'),
      '#title_display' => 'invisible',
      '#default_value' => $product->model->value,
      '#size' => 6,
    );
    $form['products'][$i]['weight'] = array(
      '#type' => 'textfield',
      '#title' => t('Weight'),
      '#title_display' => 'invisible',
      '#default_value' => $product->weight->value,
      '#size' => 3,
    );
    $units = array(
      'lb' => t('Pounds'),
      'kg' => t('Kilograms'),
      'oz' => t('Ounces'),
      'g'  => t('Grams'),
    );
    $form['products'][$i]['weight_units'] = array(
      '#type' => 'select',
      '#title' => t('Units'),
      '#title_display' => 'invisible',
      '#default_value' => $product->weight->units,
      '#options' => $units,
    );
    $form['products'][$i]['cost'] = array(
      '#type' => 'uc_price',
      '#title' => t('Cost'),
      '#title_display' => 'invisible',
      '#default_value' => $product->cost->value,
      '#size' => 5,
    );
    $form['products'][$i]['price'] = array(
      '#type' => 'uc_price',
      '#title' => t('Price'),
      '#title_display' => 'invisible',
      '#default_value' => $product->price->value,
      '#size' => 5,
    );
    $form['data'][$i]['data'] = array(
      '#type' => 'hidden',
      '#value' => serialize($product->data->first() ? $product->data->first()->toArray() : NULL),
    );
  }

  return $form;
}

/**
 * Sets the order pane to show the product selection form.
 */
function uc_order_pane_products_select($form, FormStateInterface $form_state) {
  $types = uc_product_types();
  $options = array();

  $query = db_select('node_field_data', 'n')
    ->fields('n', array('nid', 'title'))
    ->condition('n.type', $types, 'IN')
    ->orderBy('n.title')
    ->addTag('node_access');

  if (!$form_state->isValueEmpty(['product_controls', 'product_search'])) {
    $search = strtolower(str_replace('*', '%', $form_state->getValue(['product_controls', 'product_search'])));
    $query->leftJoin('uc_products', 'p', 'n.nid = p.nid');
    $query->condition(db_or()
            ->condition('n.title', $search, 'LIKE')
            ->condition('p.model', $search, 'LIKE')
          );
  }

  $result = $query->execute();
  foreach ($result as $row) {
    $options[$row->nid] = $row->title;
  }

  if (count($options) == 0) {
    $options[0] = t('No products found.');
  }

  $form_state->set('products_action', 'select');
  $form_state->set('product_select_options', $options);
  $form_state->set('refresh_products', NULL);
  $form_state->setRebuild();
}

/**
 * Sets the order pane to show the add product to order form.
 */
function uc_order_pane_products_add($form, FormStateInterface $form_state) {
  $form_state->set('products_action', 'add_product');
  $form_state->set('node', node_load($form_state->getValue(['product_controls', 'nid'])));
  $form_state->set('refresh_products', NULL);
  $form_state->setRebuild();
}

/**
 * Hides the form to add another product to the order.
 */
function uc_order_pane_products_close($form, FormStateInterface $form_state) {
  $form_state->set('products_action', NULL);
  $form_state->set('refresh_products', NULL);
  $form_state->set('product_select_options', NULL);
  $form_state->setRebuild();
}

/**
 * Form submit callback: add a blank line product to an order.
 */
function uc_order_edit_products_add_blank($form, FormStateInterface $form_state) {
  $form_state->set('refresh_products', TRUE);
  $form_state->setRebuild();

  $order = $form['#order'];

  $product = OrderProduct::create(array(
    'qty' => 1,
    'order_id' => $order->id(),
  ));
  $product->save();

  $order->products[] = $product;
  $order->logChanges(array('add' => t('Added new product line to order.')));
}

/**
 * Form submit callback: add a product to an order.
 */
function uc_order_edit_products_add($form, FormStateInterface $form_state) {
  $form_state->set('products_action', 'products_select');
  $form_state->set('refresh_products', TRUE);
  $form_state->setRebuild();

  $order = $form['#order'];

  $data = \Drupal::moduleHandler()->invokeAll('uc_add_to_cart_data', array($form_state->getValue('product_controls')));
  $values = uc_product_load_variant(intval($form_state->getValue(['product_controls', 'nid'])), $data)->toArray();
  $values['qty'] = $form_state->getValue(['product_controls', 'qty']);
  $values['order_id'] = $order->id();

  $product = OrderProduct::create($values);
  \Drupal::moduleHandler()->alter('uc_order_product', $product, $order);
  $product->save();

  $order->products[] = $product;
  $order->logChanges([t('Added (@qty) @title to order.', array('@qty' => $product->qty->value, '@title' => $product->title->value))]);

  // Decrement stock.
  if (\Drupal::moduleHandler()->moduleExists('uc_stock')) {
    uc_stock_adjust_product_stock($product, 0, $order);
  }

  // Add this product to the form values for accurate tax calculations.
  $products = $form_state->getValue('products');
  $products[] = $product;
  $form_state->setValue('products', $products);
}

/**
 * Form submit callback: remove a product from an order.
 */
function uc_order_edit_products_remove($form, FormStateInterface $form_state) {
  $form_state->set('refresh_products', TRUE);

  /** @var \Drupal\uc_order\Entity\Order $order */
  $order = $form['#order'];

  $triggering_element = $form_state->getTriggeringElement();
  $order_product_id = intval($triggering_element['#return_value']);
  /** @var \Drupal\uc_order\Entity\OrderProduct $product */
  $product = $order->products[$order_product_id];

  if (\Drupal::moduleHandler()->moduleExists('uc_stock')) {
    // Replace stock immediately.
    uc_stock_adjust($product->model->value, $product->qty->value);
  }

  $product->delete();
  unset($order->products[$order_product_id]);
  $order->logChanges([t('Removed @title from order.', array('@title' => $product->title->value))]);
}

/**
 * AJAX callback to render the order product controls.
 */
function uc_order_pane_products_ajax_callback($form, FormStateInterface $form_state) {
  $response = new AjaxResponse();
  $response->addCommand(new ReplaceCommand('#product-controls', trim(drupal_render($form['product_controls']))));
  $status_messages = array('#type' => 'status_messages');
  $response->addCommand(new PrependCommand('#product-controls', drupal_render($status_messages)));

  if ($form_state->get('refresh_products')) {
    $response->addCommand(new ReplaceCommand('#order-edit-products', trim(drupal_render($form['products']))));
    $response->addCommand(new ReplaceCommand('#order-line-items', trim(drupal_render($form['line_items']))));
  }

  // Remove the field so we only refresh the admin comments item-list.
  unset($form['admin_comment_field']);
  $response->addCommand(new ReplaceCommand('#order-pane-admin_comments .item-list', uc_order_pane_admin_comments('edit-theme', $form['#order'], $form, $form_state)));

  return $response;
}

/**
 * Handles the "Line Items" order pane.
 */
function uc_order_pane_line_items($op, $order, &$form = NULL, FormStateInterface $form_state = NULL) {
  switch ($op) {
    case 'view':
    case 'customer':
      $line_items = $order->line_items;
      $items = _uc_line_item_list();
      foreach ($items as $item) {
        if (isset($item['display_only']) && $item['display_only'] == TRUE) {
          $result = $item['callback']('display', $order);
          if (is_array($result)) {
            foreach ($result as $line) {
              $line_items[] = array(
                'title' => $line['title'],
                'amount' => $line['amount'],
                'weight' => $item['weight']
              );
            }
          }
        }
      }
      usort($line_items, 'Drupal\Component\Utility\SortArray::sortByWeightElement');

      $build['line_items'] = array(
        '#prefix' => '<table class="line-item-table">',
        '#suffix' => '</table>',
      );
      foreach ($line_items as $item) {
        $table_row = array(
          '#prefix' => '<tr>',
          '#suffix' => '</tr>',
        );

        $table_row['title'] = array(
          '#markup' => $item['title'],
          '#prefix' => '<td class="li-title">',
          '#suffix' => '</td>',
        );

        $table_row['amount'] = array(
          '#theme' => 'uc_price',
          '#price' => $item['amount'],
          '#prefix' => '<td class="li-amount">',
          '#suffix' => '</td>',
        );

        $build['line_items'][] = $table_row;
      }

      return $build;

    case 'edit-form':
      $options = array();
      $items = _uc_line_item_list();
      $line_items = $order->line_items;
      foreach ($items as $item) {
        if (isset($item['add_list']) && $item['add_list'] === TRUE) {
          $options[$item['id']] = SafeMarkup::checkPlain($item['title']);
        }
        if (isset($item['display_only']) && $item['display_only'] == TRUE) {
          $result = $item['callback']('display', $order);
          if (is_array($result)) {
            foreach ($result as $line) {
              $line_items[] = array(
                'line_item_id' => $line['id'],
                'title' => $line['title'],
                'amount' => $line['amount'],
                'weight' => $item['weight'],
              );
            }
          }
        }
      }
      usort($line_items, 'Drupal\Component\Utility\SortArray::sortByWeightElement');

      $form['add_line_item'] = array('#type' => 'container');

      $form['add_line_item']['li_type_select'] = array(
        '#type' => 'select',
        '#title' => t('Add a line item'),
        '#options' => $options,
      );
      $form['add_line_item']['submit'] = array(
        '#type' => 'submit',
        '#value' => t('Add line'),
        '#submit' => array('uc_order_pane_line_items_submit', 'uc_order_pane_line_items_add'),
        '#ajax' => array(
          'callback' => 'uc_order_pane_line_items_update',
        ),
      );
      $form['line_items'] = array(
        '#tree' => TRUE,
        '#theme' => 'uc_order_pane_line_items',
        '#prefix' => '<div id="order-line-items">',
        '#suffix' => '</div>',
      );

      foreach ($line_items as $item) {
        $form['line_items'][$item['line_item_id']]['li_id'] = array(
          '#type' => 'hidden',
          '#value' => $item['line_item_id'],
        );
        if (isset($item['type']) && _uc_line_item_data($item['type'], 'stored') == TRUE) {
          $form['line_items'][$item['line_item_id']]['remove'] = array(
            '#type' => 'image_button',
            '#title' => t('Remove line item.'),
            '#src' => drupal_get_path('module', 'uc_store') . '/images/error.gif',
            '#button_type' => 'remove',
            '#submit' => array('uc_order_pane_line_items_submit', 'uc_order_pane_line_items_remove'),
            '#ajax' => array(
              'callback' => 'uc_order_pane_line_items_update',
            ),
            '#return_value' => $item['line_item_id'],
          );
          $form['line_items'][$item['line_item_id']]['title'] = array(
            '#type' => 'textfield',
            '#title' => t('Title'),
            '#default_value' => $item['title'],
            '#size' => 40,
            '#maxlength' => 128,
          );
          $form['line_items'][$item['line_item_id']]['amount'] = array(
            '#type' => 'uc_price',
            '#title' => t('Amount'),
            '#default_value' => $item['amount'],
            '#size' => 6,
            '#allow_negative' => TRUE,
          );
        }
        else {
          $form['line_items'][$item['line_item_id']]['title'] = array(
            '#markup' => $item['title'],
          );
          $form['line_items'][$item['line_item_id']]['amount'] = array(
            '#theme' => 'uc_price',
            '#price' => $item['amount'],
          );
        }
      }
      return $form;

    case 'edit-theme':
      return drupal_render($form['add_line_item'])
           . drupal_render($form['line_items']);

    case 'edit-process':
      uc_order_pane_line_items_submit($form, $form_state);
      return;
  }
}

/**
 * @ingroup themeable
 */
function theme_uc_order_pane_line_items(array $variables) {
  $form = $variables['form'];

  $output = '<table class="line-item-table">';
  foreach (Element::children($form) as $field) {
    $form[$field]['title']['#title'] = '';
    $form[$field]['amount']['#title'] = '';
    $output .= '<tr><td class="li-title">'
      . drupal_render($form[$field]['li_id'])
      . drupal_render($form[$field]['remove'])
      . drupal_render($form[$field]['title'])
      . ':</td><td class="li-amount">'
      . drupal_render($form[$field]['amount'])
      . '</td></tr>';
  }
  $output .= '</table>' . drupal_render_children($form);

  return $output;
}

/**
 * Form submit callback: Update line items titles and amounts in an order.
 */
function uc_order_pane_line_items_submit($form, FormStateInterface $form_state) {
  $values = $form_state->getValues();

  if (is_array($values['line_items'])) {
    foreach ($values['line_items'] as $line) {
      if (is_numeric($line['li_id']) && intval($line['li_id']) > 0 && isset($line['title']) && isset($line['amount'])) {
        uc_order_update_line_item($line['li_id'], $line['title'], $line['amount']);
      }
    }
  }
}

/**
 * Order pane submit callback: Add a line item to an order.
 */
function uc_order_pane_line_items_add($form, FormStateInterface $form_state) {
  $order = &$form_state->get('order');
  $type = $form_state->getValue('li_type_select');

  uc_order_line_item_add($order->id(), $type, _uc_line_item_data($type, 'title'), 0);
  $order->line_items = $order->getLineItems();

  $form_state->setRebuild();
}

/**
 * Order pane submit callback: Remove a line item from an order.
 */
function uc_order_pane_line_items_remove($form, FormStateInterface $form_state) {
  $order = &$form_state->get('order');
  $triggering_element = $form_state->getTriggeringElement();
  $line_item_id = intval($triggering_element['#return_value']);

  uc_order_delete_line_item($line_item_id);
  $order->line_items = $order->getLineItems();

  $form_state->setRebuild();
}

/**
 * AJAX callback to render the line items.
 */
function uc_order_pane_line_items_update($form, FormStateInterface $form_state) {
  $response = new AjaxResponse();
  $response->addCommand(new ReplaceCommand('#order-line-items', trim(drupal_render($form['line_items']))));
  $status_messages = array('#type' => 'status_messages');
  $response->addCommand(new PrependCommand('#order-line-items', drupal_render($status_messages)));

  return $response;
}

/**
 * Handles the "Order Comments" order pane.
 */
function uc_order_pane_order_comments($op, $order, &$form = NULL, FormStateInterface $form_state = NULL) {
  switch ($op) {
    case 'view':
      $build = array(
        '#theme' => 'table',
        '#header' => array(
          array('data' => t('Date'), 'class' => array('date')),
          array('data' => t('User'), 'class' => array('user')),
          array('data' => t('Notified'), 'class' => array('notified')),
          array('data' => t('Status'), 'class' => array('status')),
          array('data' => t('Comment'), 'class' => array('message')),
        ),
        '#rows' => array(),
        '#attributes' => array('class' => array('order-pane-table uc-order-comments')),
        '#empty' => t('This order has no comments associated with it.'),
      );
      $comments = uc_order_comments_load($order->id());
      $statuses = OrderStatus::loadMultiple();
      foreach ($comments as $comment) {
        $icon = $comment->notified ? 'true-icon.gif' : 'false-icon.gif';
        $build['#rows'][] = array(
          array('data' => \Drupal::service('date.formatter')->format($comment->created, 'short'), 'class' => array('date')),
          array('data' => array('#theme' => 'uc_uid', '#uid' => $comment->uid), 'class' => array('user')),
          array('data' => array('#theme' => 'image', '#uri' => drupal_get_path('module', 'uc_order') . '/images/' . $icon), 'class' => array('notified')),
          array('data' => SafeMarkup::checkPlain($statuses[$comment->order_status]->getName()), 'class' => array('status')),
          array('data' => SafeMarkup::checkPlain($comment->message), 'class' => array('message')),
        );
      }
      return $build;

    case 'customer':
      $comments = uc_order_comments_load($order->id());
      $statuses = OrderStatus::loadMultiple();
      $header = array(t('Date'), t('Status'), t('Message'));
      $rows[] = array(
        array('data' => \Drupal::service('date.formatter')->format($order->created->value, 'uc_store'), 'class' => array('date')),
        array('data' => '-', 'class' => array('status')),
        array('data' => t('Order created.'), 'class' => array('message')),
      );
      if (count($comments) > 0) {
        foreach ($comments as $comment) {
          $rows[] = array(
            array('data' => \Drupal::service('date.formatter')->format($comment->created, 'uc_store'), 'class' => array('date')),
            array('data' => SafeMarkup::checkPlain($statuses[$comment->order_status]->getName()), 'class' => array('status')),
            array('data' => SafeMarkup::checkPlain($comment->message), 'class' => array('message')),
          );
        }
      }
      $build = array(
        '#theme' => 'table',
        '#header' => $header,
        '#rows' => $rows,
        '#attributes' => array('class' => array('uc-order-comments')),
      );

      return $build;
  }
}

/**
 * Handles the "Admin Comments" order pane.
 */
function uc_order_pane_admin_comments($op, $order, &$form = NULL, FormStateInterface $form_state = NULL) {
  switch ($op) {
    case 'view':
      $build = array(
        '#theme' => 'table',
        '#header' => array(
          array('data' => t('Date'), 'class' => array('date')),
          array('data' => t('User'), 'class' => array('user')),
          array('data' => t('Comment'), 'class' => array('message')),
        ),
        '#rows' => array(),
        '#attributes' => array('class' => array('order-pane-table uc-order-comments')),
        '#empty' => t('This order has no admin comments associated with it.'),
      );
      $comments = uc_order_comments_load($order->id(), TRUE);
      foreach ($comments as $comment) {
        $build['#rows'][] = array(
          array('data' => \Drupal::service('date.formatter')->format($comment->created, 'short'), 'class' => array('date')),
          array('data' => array('#theme' => 'uc_uid', '#uid' => $comment->uid), 'class' => array('user')),
          array('data' => Xss::filterAdmin($comment->message), 'class' => array('message')),
        );
      }
      return $build;

    case 'edit-form':
      $form['admin_comment_field'] = array(
        '#type' => 'details',
        '#title' => t('Add an admin comment'),
      );
      $form['admin_comment_field']['admin_comment'] = array(
        '#type' => 'textarea',
        '#description' => t('Admin comments are only seen by store administrators.'),
      );
      return $form;

    case 'edit-theme':
      $items = array();
      $comments = uc_order_comments_load($form['order_id']['#value'], TRUE);
      foreach ($comments as $comment) {
        $items[] = [
          'username' => [
            '#theme' => 'uc_uid',
            '#uid' => $comment->uid,
            '#prefix' => '[',
            '#suffix' => '] ',
          ],
          'message' => [
            '#markup' => $comment->message,
          ]
        ];
      }
      $item_list = array(
        '#theme' => 'item_list',
        '#items' => $items,
        '#empty' => t('No admin comments have been entered for this order.'),
      );
      $output = drupal_render($item_list);
      $output .= drupal_render($form['admin_comment_field']);
      return $output;

    case 'edit-process':
      if (!$form_state->isValueEmpty('admin_comment')) {
        $uid = \Drupal::currentUser()->id();
        uc_order_comment_save($form_state->getValue('order_id'), $uid, $form_state->getValue('admin_comment'));
      }
      return;
  }
}

/**
 * Handles the "Update" order pane.
 */
function uc_order_pane_update($op, $order, &$form = NULL, FormStateInterface $form_state = NULL) {
  switch ($op) {
    case 'view':
      return \Drupal::formBuilder()->getForm('\Drupal\uc_order\Form\OrderUpdateForm', $order);
  }
}

/**
 * Builds a list of order panes defined in the enabled modules.
 */
function _uc_order_pane_list($view = 'view') {
  static $panes = array();

  if (count($panes) > 0) {
    return $panes;
  }

  foreach (\Drupal::moduleHandler()->invokeAll('uc_order_pane') as $id => $pane) {
    // Preserve backward compatibility for panes with no key specified.
    if (is_numeric($id)) {
      $id = $pane['id'];
    }

    // Set defaults.
    $pane += array(
      'id' => $id,
      'enabled' => TRUE,
      'weight' => 0,
    );

    $panes[$id] = $pane;
  }

  // Allow other modules to alter the defaults.
  \Drupal::moduleHandler()->alter('uc_order_pane', $panes);

  uasort($panes, 'Drupal\Component\Utility\SortArray::sortByWeightElement');

  return $panes;
}
